Package: sfheaders
Type: Package
Title: Converts Between R Objects and Simple Feature Objects
Date: 2024-01-18
Version: 0.4.4
Authors@R: c(
    person("David", "Cooley", ,"david.cooley.au@gmail.com", role = c("aut", "cre")),
    person(given = "Michael", family = "Sumner", role = "ctb")
    )
Description: Converts between R and Simple Feature 'sf' objects, without depending
  on the Simple Feature library. Conversion functions are available at both the R level, 
  and through 'Rcpp'.
License: MIT + file LICENSE
URL: https://dcooley.github.io/sfheaders/
BugReports: https://github.com/dcooley/sfheaders/issues
Encoding: UTF-8
RoxygenNote: 7.2.3
Depends: R (>= 3.0.2)
LinkingTo: geometries (>= 0.2.4), Rcpp
Imports: Rcpp (>= 1.0.10)
Suggests: covr, knitr, testthat
NeedsCompilation: yes
Packaged: 2024-01-17 21:15:30 UTC; dave
Author: David Cooley [aut, cre],
  Michael Sumner [ctb]
Maintainer: David Cooley <david.cooley.au@gmail.com>
Repository: CRAN
Date/Publication: 2024-01-17 21:50:02 UTC
Built: R 4.4.0; x86_64-apple-darwin16.7.0; 2024-04-29 17:36:40 UTC; unix
