Package: sfheaders
Type: Package
Title: Converts Between R Objects and Simple Feature Objects
Date: 2020-05-15
Version: 0.2.2
Authors@R: c(
    person("David", "Cooley", ,"david.cooley.au@gmail.com", role = c("aut", "cre")),
    person(given = "Michael", family = "Sumner", role = "ctb")
    )
Description: Converts between R and Simple Feature 'sf' objects, without depending
  on the Simple Feature library. Conversion functions are available at both the R level, 
  and through 'Rcpp'.
License: GPL-3
URL: https://dcooley.github.io/sfheaders/
BugReports: https://github.com/dcooley/sfheaders/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.0
SystemRequirements: C++11
LinkingTo: Rcpp
Imports: Rcpp
Suggests: covr, knitr, testthat
NeedsCompilation: yes
Packaged: 2020-05-13 05:32:06 UTC; david
Author: David Cooley [aut, cre],
  Michael Sumner [ctb]
Maintainer: David Cooley <david.cooley.au@gmail.com>
Repository: CRAN
Date/Publication: 2020-05-13 06:30:02 UTC
Built: R 4.0.0; x86_64-w64-mingw32; 2020-05-13 07:43:57 UTC; windows
Archs: x64
