# Generated by roxygen2: do not edit by hand

S3method(calculate_bbox,default)
S3method(calculate_bbox,sf)
S3method(calculate_boxes,sf)
S3method(calculate_boxes,sfc)
S3method(calculate_boxes,sfg)
S3method(remove_holes,default)
S3method(remove_holes,sf)
S3method(remove_holes,sfc)
S3method(remove_holes,sfg)
export(sf_bbox)
export(sf_boxes)
export(sf_cast)
export(sf_line)
export(sf_linestring)
export(sf_mline)
export(sf_mpoly)
export(sf_mpt)
export(sf_multilinestring)
export(sf_multipoint)
export(sf_multipolygon)
export(sf_point)
export(sf_poly)
export(sf_polygon)
export(sf_pt)
export(sf_remove_holes)
export(sf_to_df)
export(sfc_cast)
export(sfc_linestring)
export(sfc_multilinestring)
export(sfc_multipoint)
export(sfc_multipolygon)
export(sfc_point)
export(sfc_polygon)
export(sfc_to_df)
export(sfg_linestring)
export(sfg_multilinestring)
export(sfg_multipoint)
export(sfg_multipolygon)
export(sfg_point)
export(sfg_polygon)
export(sfg_to_df)
importFrom(Rcpp,sourceCpp)
useDynLib(sfheaders, .registration = TRUE)
